#include <cdcacm.h>
#include <usbhub.h>
#include <RF24Network.h>   // отсюда: https://github.com/maniacbug/RF24Network
#include <RF24.h>          // отсюда: https://github.com/maniacbug/RF24
#include <SPI.h>
#include "printf.h"

int USB5V = 7; // подключен к USB +5V
RF24 radio(9,10);          // !!!!
RF24Network network(radio);
uint16_t this_node = 013;  // Адрес нашего узла  !!!!
const unsigned long interval = 25000; // мс
unsigned long last_time_sent;

// Массив с известными нам узлами
const short max_active_nodes = 10;
uint16_t active_nodes[max_active_nodes];
short num_active_nodes = 0;
short next_ping_node_index = 0;
// Описание функций отправки и обработки пакетов
bool send_T(uint16_t to);
bool send_N(uint16_t to);
bool send_D(uint16_t to);
void handle_T(RF24NetworkHeader& header);
void handle_N(RF24NetworkHeader& header);
void handle_D(RF24NetworkHeader& header);
void add_node(uint16_t node);

uint8_t rcode;
const char *messageUSB;
int messagelen;

class ACMAsyncOper : 
public CDCAsyncOper
{
public:
  virtual uint8_t OnInit(ACM *pacm);
};
uint8_t ACMAsyncOper::OnInit(ACM *pacm)
{
  uint8_t rcode;
  rcode = pacm->SetControlLineState(3);
  if (rcode)
  {
    ErrorMessage<uint8_t>(PSTR("SetControlLineState"), rcode);
    return rcode;
  }
  LINE_CODING	lc;
  lc.dwDTERate	= 115200;
  lc.bCharFormat	= 0;
  lc.bParityType	= 0;
  lc.bDataBits	= 8;
  rcode = pacm->SetLineCoding(&lc);
  if (rcode)
    ErrorMessage<uint8_t>(PSTR("SetLineCoding"), rcode);
  return rcode;
}
USB     Usb;
ACMAsyncOper  AsyncOper;
ACM           Acm(&Usb, &AsyncOper);

void setup()
{
  pinMode(USB5V, OUTPUT);
  digitalWrite(USB5V, LOW);
  Serial.begin( 115200 );
  while (!Serial); 
  Serial.println("Start");
  if (Usb.Init() == -1)
    Serial.println("OSCOKIRQ failed to assert");
  delay( 200 );
  if (rcode)
    ErrorMessage<uint8_t>(PSTR("SndData"), rcode);

  SPI.begin();
  radio.begin();
  printf_begin();
  network.begin(/*канал*/ 98, /*адрес этого узла*/ this_node ); //  !!!!

  delay(100);
  RF24NetworkHeader header1(/*Кому*/ 00, /*Тип*/ 'D'); //reset Aqua
  unsigned long message = 913 ;
  network.write(header1,&message,sizeof(message));
  Serial.println("Send 913"); 
}

void loop(){
  network.update();    // Регулярно вызываем обработчик пакетов в сети. Это обязательно!
  while (network.available())   // Есть в эфире что нибудь для нас?
  {
    // Если есть, то посмотрим что там пришло
    RF24NetworkHeader header;
    network.peek(header);
    // Передаем сообщение нужному обработчику, в зависимости от заголовка в пакете.
    switch (header.type)
    {
    case 'T':
      handle_T(header);
      break;
    case 'N':
      handle_N(header);
      break;
    case 'D':
      handle_D(header);
      break;
    default:
      printf_P(PSTR("*** WARNING *** Unknown message type %c\n\r"),header.type);
      network.read(header,0,0);
      break;
    };
  }

  // Отправляем ping активным узлам каждые 'interval' мс
  unsigned long now = millis();
  if ( now - last_time_sent >= interval )  {
    last_time_sent = now;

    // Кому мы должны отправить?
    uint16_t to = 00;                                   // По умолчанию отправим на базу
    if (num_active_nodes) {                             // Или если у нас есть активные узлы,
      to = active_nodes[next_ping_node_index++];        // отправим на след. активный узел
      if ( next_ping_node_index > num_active_nodes ) {  // Если перебор узлов закончен
        next_ping_node_index = 0;                       // то в следующий раз начнем с начала
        to = 00;                                        // а сейчас отправим на узел 00.
      }
    }
    bool ok;
    // Если этот узел является обычным, то отприм пакет типа 'T' (ping)
    if ( this_node > 00 || to == 00 )
      ok = send_T(to);
    // Елси он базовый, то будем отправлять список всех активных узлов всем
    else
      ok = send_N(to);
    // Сообщаем о результатах
    if (ok) {
      Serial.println("SendC"); 
    }
    else    {
      //      printf_P(PSTR("%lu: APP Send failed\n\r"),millis());
      last_time_sent -= 100; // В следующий раз попробуем отправить на 100мс раньше
    }
  }
}

int Send_Message(){
  while (messageUSB != ""){
    Usb.Task();
    if( Acm.isReady()) {
      messagelen = strlen(messageUSB);
      for (int i = 0; i < messagelen; i++){
        uint8_t data = messageUSB[i];
        rcode = Acm.SndData(1, &data);
      }
      //  Serial.println("      Send Message Complete");
      delay(50);
      uint8_t  buf[64];
      uint16_t rcvd = 64;
      rcode = Acm.RcvData(&rcvd, buf);
      if( rcvd ) { //more than zero bytes received
        for(uint16_t i=0; i < rcvd; i++ ) {
          Serial.print((char)buf[i]); //printing on the screen
        }
      }
      messageUSB= "" ;
    }
  }
}


//__________________________________отправка сообщений___________________________________________
// Отправка пинга. Тут будем отправлять пакет с меткой 'T', это просто переменная ulong, содержащая время millis()
bool send_T(uint16_t to) {
  RF24NetworkHeader header(/*Кому*/ to, /*Тип*/ 'T' /*Time*/);
  unsigned long message = millis();
  return network.write(header,&message,sizeof(unsigned long));
}
//Отправка пакета с типом метки 'N', это список активных узлов (таблица)
bool send_N(uint16_t to) {
  RF24NetworkHeader header(/*Кому*/ to, /*Тип*/ 'N' /*Nodes*/);
  return network.write(header,active_nodes,sizeof(active_nodes));
}
//__________________________________функции обработки приенятых сообщений____________________________
// Обработчик полученных сообщений с заголовком типа 'T'. Просто добавляем узел в список активных узлов
void handle_T(RF24NetworkHeader& header) {
  unsigned long message;
  network.read(header,&message,sizeof(unsigned long));
  if ( header.from_node != this_node || header.from_node > 00 )
    add_node(header.from_node);
}
// Обработчик сообщений с заголовком типа 'N' - список активных узлов. Обновляем свой список узлов.
void handle_N(RF24NetworkHeader& header) {
  static uint16_t incoming_nodes[max_active_nodes];
  network.read(header,&incoming_nodes,sizeof(incoming_nodes));
  int i = 0;
  while ( i < max_active_nodes && incoming_nodes[i] > 00 )
    add_node(incoming_nodes[i++]);
}
// Обработчик сообщений с заголовком типа 'D' - список активных узлов. Обновляем свой список узлов.
void handle_D(RF24NetworkHeader& header) {
  unsigned long message;
  network.read(header,&message,sizeof(unsigned long));
  Serial.println(message);

  if (message == 000){            // Ping
    RF24NetworkHeader header1(/*Кому*/ 00, /*Тип*/ 'D'); //reset Aqua
    unsigned long message = 013 ;
    network.write(header1,&message,sizeof(message));
  }
  if (message == 222 ){           //Clean House
    digitalWrite(USB5V, HIGH);
    delay(500);
    messageUSB = "PlaySound 2\r";
    Send_Message();
    delay(2000);
    messageUSB = "Clean House\r";
    Send_Message();
    delay(2000);
    messageUSB = "Clean House\r";
    Send_Message();
    digitalWrite(USB5V, LOW);
  }
  if (message == 333 ){           //Очистка фильтра >>40СМ
    digitalWrite(USB5V, HIGH);
    delay(500);
    messageUSB = "PlaySound 2\r";
    Send_Message();
    delay(1000);
    messageUSB = "TestMode On\r";
    Send_Message();
    delay(1000);
    messageUSB = "setmotor 400 400 100\r";
    Send_Message();  
    delay(4000);  
    messageUSB = "TestMode Off\r";
    Send_Message();
    delay(500);
    messageUSB = "PlaySound 2\r";
    Send_Message();
    digitalWrite(USB5V, LOW);
  }
  if (message == 444 ){           //Clean Spot
    digitalWrite(USB5V, HIGH);
    delay(500);
    messageUSB = "PlaySound 2\r";
    Send_Message();
    delay(2000);
    messageUSB = "Clean Spot\r";
    Send_Message();
    delay(2000);
    messageUSB = "Clean Spot\r";
    Send_Message();
    digitalWrite(USB5V, LOW);
  }
  if (message == 111 ){           //Clean Stop
    digitalWrite(USB5V, HIGH);
    delay(500);
    messageUSB = "Clean Stop\r";
    Send_Message();
    delay(200);
    messageUSB = "PlaySound 2\r";
    Send_Message();
    digitalWrite(USB5V, LOW);
  }
  if ( header.from_node != this_node || header.from_node > 00 )
    add_node(header.from_node);
}
//__________________________________работа со списком активных узлов____________________________
void add_node(uint16_t node) {
  short i = num_active_nodes;
  while (i--) {
    if (active_nodes[i] == node)
      break;
  }
  if (i == -1 && num_active_nodes < max_active_nodes) {
    active_nodes[num_active_nodes++] = node;
  }
} 

